

DELIMITER //


DROP PROCEDURE IF EXISTS sp_purge_execution_jobs_delete_chunk
//
CREATE PROCEDURE sp_purge_execution_jobs_delete_chunk(
	IN p_sql_part varchar(1000),
	IN p_delete_chunk_size int,
	IN p_time_delay int,
	IN p_p1 datetime, 
	IN p_tablename varchar(30)
	)
BEGIN
	DECLARE l_rows_deleted int;
	-- DECLARE l_sql varchar(1000);
	DECLARE l_delay_stmt varchar(500);

	SET @sql = concat(p_sql_part, ' LIMIT ', p_delete_chunk_size);
	-- select @sql;
	
	set l_rows_deleted = p_delete_chunk_size;
	set @p1 = p_p1;

    PREPARE stmt FROM @sql;
    
	WHILE l_rows_deleted = p_delete_chunk_size DO
    	EXECUTE stmt USING @p1;
    	SELECT ROW_COUNT() INTO l_rows_deleted;
    	select concat(l_rows_deleted, ' rows were deleted from table: ', p_tablename, ' for jobs that finished on ', DATE(DATE_ADD(p_p1, INTERVAL -1 DAY))) as rows_deleted;
    	IF p_time_delay IS NOT NULL THEN
    		DO SLEEP(p_time_delay);
    	END IF;
    END WHILE;

    DEALLOCATE PREPARE stmt;

END//



DROP PROCEDURE IF EXISTS sp_purge_execution_jobs
//
CREATE PROCEDURE sp_purge_execution_jobs(
	IN p_retention_days int,
	IN p_time_limit int,
	IN p_time_delay int,
	IN p_delete_chunk_size int
	)
BEGIN
	DECLARE l_cur_date datetime;
	DECLARE l_cutoff_date datetime;
	DECLARE l_stop_time datetime;
	DECLARE l_sql varchar(500);
	
	-- set default values
	IF p_retention_days is null THEN SET p_retention_days = 90; END IF;
	IF p_delete_chunk_size is null THEN SET p_delete_chunk_size = 10000; END IF;

	
	-- set l_stop_time
	CASE 
		WHEN p_time_limit is null THEN set l_stop_time = null; 
		ELSE SET l_stop_time = DATE_ADD(now(), INTERVAL p_time_limit SECOND); 
	END CASE;

	-- select concat('l_stop_time = ', ifnull(l_stop_time, 'null'));
	select concat('Purging Offline Execution Job tables. Will stop purging at: ', ifnull(l_stop_time, 'null')) as Status;

    
    -- set l_cur_date
    select DATE_ADD(min(end_time), INTERVAL 1 day) into l_cur_date from offline_execution_jobs where end_time is not null; 
	

  	
  	-- set l_cutoff_date
  	SET l_cutoff_date = DATE_ADD(now(), INTERVAL - p_retention_days DAY);
	-- select concat('l_cutoff_date = ', ifnull(l_cutoff_date, 'null'));


  	-- Each iteration of the loop processes a day's worth of offline data
	WHILE l_cur_date < l_cutoff_date AND (l_stop_time is null OR now() < l_stop_time)
	DO
		select concat('Purging offline execution job data for jobs that completed on: ', ifnull(DATE(DATE_ADD(l_cur_date, INTERVAL -1 DAY)), 'null')) as purge_date;
		
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_distribution_events where id in (select e.id from offline_execution_jobs j, offline_flow_events e where j.id = e.job_id and j.end_time < ?)', 
			p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_distribution_events');

		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_exec_param_links where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_exec_param_links');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_exec_servers where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_exec_servers');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_flow_events where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_flow_events');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_manual_operations where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_manual_operations');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_parameter_requests where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_parameter_requests');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_parameters where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_parameters');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_propagation_events where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_propagation_events');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_step_events where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_step_events');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_steps where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_steps');
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_user_events where job_id in (select id from offline_execution_jobs where end_time < ?)', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_user_events');
		
		CALL sp_purge_execution_jobs_delete_chunk('delete from offline_execution_jobs where end_time < ?', p_delete_chunk_size, p_time_delay, l_cur_date, 'offline_execution_jobs');

		-- update l_cur_date
		set l_cur_date =  DATE_ADD(l_cur_date, INTERVAL 1 DAY);

	END WHILE;
	
	IF (now() > l_stop_time) THEN
    	select 'Exceeded Time limit.  Exiting procedure.' as Status;
  	ELSE
    	select 'Purge of offline execution job tables completed.' as Status;
    END IF;
END//

DELIMITER ;	




   